/*****************************************************************************
            --- EJEMPLO de uso del importador de mallas de OGRE ---
------------------------------------------------------------------------------
   Autor: Jose L. Hidalgo
   Fecha: 24/04/2005

******************************************************************************/

//Headers:
#include <Ogre.h>
#include <ExampleApplication.h>
#include <OgreGTGeometry.h>


/*********************  Test Application  ************************************/

class Application : public ExampleApplication
{
protected:
	void createScene(void);
	OgreGTGeometry ogreImporter;

public:
};


void Application::createScene(void)
{

	// Original mesh
	Ogre::String meshName("ogrehead.mesh");
	Ogre::MeshPtr meshOriginal = Ogre::MeshManager::getSingleton().
		load(meshName, Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
	
	// TRANSFORM !!!!
	Geometry::Mesh *geoMesh = OgreGTGeometry::getSingleton().transform(meshOriginal.getPointer());
	Ogre::MeshPtr  ogreMesh = OgreGTGeometry::getSingleton().transform(geoMesh, Ogre::String("nuevo"));
	
	
	// Attach objects:
	Ogre::Entity* thisEntity1 = mSceneMgr->createEntity("blah_1",meshName);
	Ogre::Entity* thisEntity2 = mSceneMgr->createEntity("blah_2","nuevo");
	
	Ogre::SceneNode* thisSceneNode1 = static_cast<SceneNode*>(mSceneMgr->getRootSceneNode()->createChild());
	Ogre::SceneNode* thisSceneNode2 = static_cast<SceneNode*>(mSceneMgr->getRootSceneNode()->createChild());
	Ogre::Light *light = mSceneMgr->createLight("luz");

	light->setPosition(0,200,0);
	light->setDiffuseColour(Ogre::ColourValue());
	mSceneMgr->setAmbientLight(Ogre::ColourValue(0.5,0.5,0.5));
	mSceneMgr->setSkyDome(true,"Examples/CloudySky");
	
	Ogre::Vector3 pos(1,0,0);
	pos *= ogreMesh->getBoundingSphereRadius();

	thisSceneNode1->setPosition(pos);
    thisSceneNode1->attachObject(thisEntity1);

	thisSceneNode2->setPosition(-pos);
    thisSceneNode2->attachObject(thisEntity2);
}



/*****************************************************************************/
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"

INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
#else
int main(int argc, char **argv)
#endif
{
    // Create application object
    Application app;

    try {
        app.go();
    } catch( Exception& e ) {
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32 
        MessageBox( NULL, e.getFullDescription().c_str(), 
			"An exception has occured!", 
			MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
        fprintf(stderr, "An exception has occured: %s\n",
                e.getFullDescription().c_str());
#endif
    }

    return 0;
}