#include "HierarchyInterface.h"

#include <windows.h>

namespace GtpVisibility {

//-----------------------------------------------------------------------
HierarchyInterface::HierarchyInterface(): 
mFrameId(0), mNumTraversedNodes(0), mHierarchyRoot(NULL), 
mSavedNode(NULL), mCurrentTestIdx(0), mTestGeometryForVisibleLeaves(false)
{	
	mDistanceQueue = new DistanceQueue(GreaterDistance<HierarchyNode *>(this));
	mTraversalStack = new std::stack<HierarchyNode *>;
}
//-----------------------------------------------------------------------
HierarchyInterface::~HierarchyInterface()
{		
	delete mDistanceQueue;
	delete mTraversalStack;
}
//-----------------------------------------------------------------------
void HierarchyInterface::SetHierarchyRoot(HierarchyNode *root)
{
	mHierarchyRoot = root;
}
//-----------------------------------------------------------------------
void HierarchyInterface::InitTraversal(bool frontToBack)
{
	// initialise for front-to-back rendering
	if (frontToBack)
	{
		++ mFrameId;
		mCurrentTestIdx = 0;
		mNumTraversedNodes = 0;
		mRenderedNodes.clear();

		mDistanceQueue->push(mHierarchyRoot);
	}
 	else
	{	// initialise for simple node traversal
		mTraversalStack->push(mHierarchyRoot);
	}
}
//-----------------------------------------------------------------------
unsigned int HierarchyInterface::GetFrameId() const
{
	return mFrameId;
}
//-----------------------------------------------------------------------
DistanceQueue *HierarchyInterface::GetQueue()
{
	return mDistanceQueue;
}
//-----------------------------------------------------------------------
bool HierarchyInterface::CheckFrustumVisible(HierarchyNode *node)
{
	bool intersects = false;
	return CheckFrustumVisible(node, intersects);
}
//-----------------------------------------------------------------------
HierarchyNode *HierarchyInterface::GetHierarchyRoot() const
{
	return mHierarchyRoot;
}
//-----------------------------------------------------------------------
unsigned int HierarchyInterface::GetNumTraversedNodes()
{
	return mNumTraversedNodes;
}
//-----------------------------------------------------------------------
unsigned int HierarchyInterface::GetNumRenderedNodes()
{
	return (unsigned int)mRenderedNodes.size();
}
//-----------------------------------------------------------------------
void HierarchyInterface::TestGeometryForVisibleLeaves(bool testGeometry)
{
	mTestGeometryForVisibleLeaves = testGeometry;
}
//-----------------------------------------------------------------------
std::vector<HierarchyNode *> *HierarchyInterface::GetRenderedNodes()
{
	return &mRenderedNodes;
}
} // namespace GtpVisibility
